@@ -3,8 +3,8 @@ use std::{fs, mem::swap, path::PathBuf, str::FromStr};
 use clap::{IntoApp, Parser};
 use espflash::{
     cli::{
-        board_info, connect, flash_elf_image, monitor::monitor, save_elf_as_image, ConnectOpts,
-        FlashConfigOpts, FlashOpts,
+        board_info, connect, flash_elf_image, monitor::monitor, partition_table, save_elf_as_image,
+        ConnectOpts, FlashConfigOpts, FlashOpts, PartitionTableOpts,
     },
     Chip, Config, ImageFormatId,
 };
@@ -34,6 +34,8 @@ pub enum SubCommand {
     BoardInfo(ConnectOpts),
     /// Save the image to disk instead of flashing to device
     SaveImage(SaveImageOpts),
+    /// Operations for partitions tables
+    PartitionTable(PartitionTableOpts),
 }
 
 #[derive(Parser)]
@@ -66,7 +68,10 @@ fn main() -> Result<()> {
     let mut opts = Opts::parse();
     let config = Config::load()?;
 
-    if !matches!(opts.subcommand, Some(SubCommand::BoardInfo(..))) {
+    if !matches!(
+        opts.subcommand,
+        Some(SubCommand::BoardInfo(..) | SubCommand::PartitionTable(..)),
+    ) {
         // If neither the IMAGE nor SERIAL arguments have been provided, print the
         // help message and exit.
         if opts.image.is_none() && opts.connect_opts.serial.is_none() {
@@ -89,6 +94,7 @@ fn main() -> Result<()> {
         match subcommand {
             BoardInfo(opts) => board_info(opts, config),
             SaveImage(opts) => save_image(opts),
+            PartitionTable(opts) => partition_table(opts),
         }
     } else {
         flash(opts, config)
